package com.sun.xml.internal.ws.addressing.policy;

import com.sun.xml.internal.ws.policy.PolicyAssertion;
import java.util.Collections;
import com.sun.xml.internal.ws.policy.AssertionSet;
import com.sun.xml.internal.ws.policy.sourcemodel.AssertionData;
import com.sun.xml.internal.ws.addressing.W3CAddressingMetadataConstants;
import com.sun.xml.internal.ws.policy.Policy;
import javax.xml.namespace.QName;
import com.sun.xml.internal.ws.policy.subject.WsdlBindingSubject;
import com.sun.xml.internal.ws.policy.PolicyException;
import java.util.logging.Level;
import javax.xml.ws.soap.AddressingFeature;
import java.util.ArrayList;
import com.sun.xml.internal.ws.policy.PolicySubject;
import java.util.Collection;
import com.sun.xml.internal.ws.api.WSBinding;
import com.sun.xml.internal.ws.api.model.SEIModel;
import com.sun.xml.internal.ws.policy.PolicyMap;
import com.sun.xml.internal.ws.policy.privateutil.PolicyLogger;
import com.sun.xml.internal.ws.policy.jaxws.spi.PolicyMapConfigurator;

public class AddressingPolicyMapConfigurator implements PolicyMapConfigurator
{
    private static final PolicyLogger LOGGER;
    
    public AddressingPolicyMapConfigurator() {
    }
    
    public Collection<PolicySubject> update(PolicyMap policyMap, SEIModel model, WSBinding wsBinding) throws PolicyException {
        AddressingPolicyMapConfigurator.LOGGER.entering(new Object[] { policyMap, model, wsBinding });
        Collection<PolicySubject> subjects = new ArrayList();
        if (policyMap != null) {
            AddressingFeature addressingFeature = (AddressingFeature)wsBinding.getFeature(AddressingFeature.class);
            if (AddressingPolicyMapConfigurator.LOGGER.isLoggable(Level.FINEST)) {
                AddressingPolicyMapConfigurator.LOGGER.finest("addressingFeature = " + addressingFeature);
            }
            if (addressingFeature != null && addressingFeature.isEnabled()) {
                this.addWsamAddressing(subjects, policyMap, model, addressingFeature);
            }
        }
        AddressingPolicyMapConfigurator.LOGGER.exiting(subjects);
        return subjects;
    }
    
    private void addWsamAddressing(Collection<PolicySubject> subjects, PolicyMap policyMap, SEIModel model, AddressingFeature addressingFeature) throws PolicyException {
        QName bindingName = model.getBoundPortTypeName();
        WsdlBindingSubject wsdlSubject = WsdlBindingSubject.createBindingSubject(bindingName);
        Policy addressingPolicy = this.createWsamAddressingPolicy(bindingName, addressingFeature);
        PolicySubject addressingPolicySubject = new PolicySubject(wsdlSubject, addressingPolicy);
        subjects.add(addressingPolicySubject);
        if (AddressingPolicyMapConfigurator.LOGGER.isLoggable(Level.FINE)) {
            AddressingPolicyMapConfigurator.LOGGER.fine("Added addressing policy with ID \"" + addressingPolicy.getIdOrName() + "\" to binding element \"" + bindingName + "\"");
        }
    }
    
    private Policy createWsamAddressingPolicy(QName bindingName, AddressingFeature af) {
        ArrayList<AssertionSet> assertionSets = new ArrayList(1);
        ArrayList<PolicyAssertion> assertions = new ArrayList(1);
        AssertionData addressingData = AssertionData.createAssertionData(W3CAddressingMetadataConstants.WSAM_ADDRESSING_ASSERTION);
        if (!af.isRequired()) {
            addressingData.setOptionalAttribute(true);
        }
        try {
            AddressingFeature.Responses responses = af.getResponses();
            if (responses == AddressingFeature.Responses.ANONYMOUS) {
                AssertionData nestedAsserData = AssertionData.createAssertionData(W3CAddressingMetadataConstants.WSAM_ANONYMOUS_NESTED_ASSERTION);
                PolicyAssertion nestedAsser = new AddressingPolicyMapConfigurator.AddressingAssertion(nestedAsserData, null);
                assertions.add(new AddressingPolicyMapConfigurator.AddressingAssertion(addressingData, AssertionSet.createAssertionSet(Collections.singleton(nestedAsser))));
            }
            else if (responses == AddressingFeature.Responses.NON_ANONYMOUS) {
                AssertionData nestedAsserData = AssertionData.createAssertionData(W3CAddressingMetadataConstants.WSAM_NONANONYMOUS_NESTED_ASSERTION);
                PolicyAssertion nestedAsser = new AddressingPolicyMapConfigurator.AddressingAssertion(nestedAsserData, null);
                assertions.add(new AddressingPolicyMapConfigurator.AddressingAssertion(addressingData, AssertionSet.createAssertionSet(Collections.singleton(nestedAsser))));
            }
            else {
                assertions.add(new AddressingPolicyMapConfigurator.AddressingAssertion(addressingData, AssertionSet.createAssertionSet(null)));
            }
        }
        catch (NoSuchMethodError e) {
            assertions.add(new AddressingPolicyMapConfigurator.AddressingAssertion(addressingData, AssertionSet.createAssertionSet(null)));
        }
        assertionSets.add(AssertionSet.createAssertionSet(assertions));
        return Policy.createPolicy(null, bindingName.getLocalPart() + "_WSAM_Addressing_Policy", assertionSets);
    }
    
    static {
        LOGGER = PolicyLogger.getLogger(AddressingPolicyMapConfigurator.class);
    }
    
    private static final class AddressingAssertion extends PolicyAssertion
    {
        AddressingAssertion(AssertionData assertionData, AssertionSet nestedAlternative) {
            super(assertionData, null, nestedAlternative);
        }
        
        AddressingAssertion(AssertionData assertionData) {
            super(assertionData, null, null);
        }
    }
}
